using System;
using System.IO;
using System.Linq;
using System.Windows.Forms;
using System.Drawing;
using System.Collections.Generic;
using Newtonsoft.Json;

WaitFormVisible = false;

// Class definition for better extensibility
class MeasureDefinition
{
    public string Report { get; set; }
    public string TableName { get; set; }
    public string MeasureName { get; set; }
    public string Expression { get; set; }
    public string FormatString { get; set; }
}

// Helper to create UI controls
TextBox CreateSearchBox(EventHandler onChange)
{
    var tb = new TextBox();
    tb.PlaceholderText = "Search by measure name or expression...";
    tb.Location = new Point(10, 10);
    tb.Size = new Size(960, 20);
    tb.TextChanged += onChange;
    return tb;
}

ListView CreateListView(List<MeasureDefinition> source, Func<string> getFilter)
{
    var lv = new ListView();
    lv.Location = new Point(10, 40);
    lv.Size = new Size(960, 330);
    lv.View = View.Details;
    lv.FullRowSelect = true;
    lv.CheckBoxes = true;
    lv.Columns.Add("Measure Name", 200);
    lv.Columns.Add("Expression", 480);
    lv.Columns.Add("Table", 150);
    lv.Columns.Add("Report", 100);

    void populate()
    {
        lv.Items.Clear();
        foreach (var m in source)
        {
            if (string.IsNullOrWhiteSpace(getFilter()) ||
                m.MeasureName.IndexOf(getFilter(), StringComparison.OrdinalIgnoreCase) >= 0 ||
                m.Expression.IndexOf(getFilter(), StringComparison.OrdinalIgnoreCase) >= 0)
            {
                var item = new ListViewItem(m.MeasureName);
                item.SubItems.Add(m.Expression);
                item.SubItems.Add(m.TableName);
                item.SubItems.Add(m.Report);
                item.Checked = true;
                lv.Items.Add(item);
            }
        }
    }
    lv.Tag = (Action)(() => populate());
    lv.ColumnClick += (s, e) => {
        if (e.Column == 0) source.Sort((a, b) => a.MeasureName.CompareTo(b.MeasureName));
        populate();
    };
    return lv;
}

// Prompt for folder
var folderBrowser = new FolderBrowserDialog();
folderBrowser.Description = "Select the root PBIP folder";
folderBrowser.ShowNewFolderButton = false;
if (folderBrowser.ShowDialog() != DialogResult.OK || string.IsNullOrWhiteSpace(folderBrowser.SelectedPath))
    throw new Exception("No folder selected.");

string pbipFolderPath = folderBrowser.SelectedPath;

// Load measures
var allMeasures = new List<MeasureDefinition>();
var reportDirs = Directory.GetDirectories(pbipFolderPath)
    .Where(d => d.EndsWith(".Report", StringComparison.OrdinalIgnoreCase));

foreach (var reportDir in reportDirs)
{
    string reportName = Path.GetFileNameWithoutExtension(reportDir).Replace(".Report", "");
    string reportExtensionsPath = Path.Combine(reportDir, "definition", "reportExtensions.json");
    if (!File.Exists(reportExtensionsPath)) continue;

    string json = File.ReadAllText(reportExtensionsPath);
    dynamic reportExtensions = JsonConvert.DeserializeObject(json);

    foreach (var entity in reportExtensions.entities)
    {
        string tableName = entity.name;
        foreach (var measureDef in entity.measures)
        {
            var md = new MeasureDefinition
            {
                Report = reportName,
                TableName = tableName,
                MeasureName = measureDef.name,
                Expression = measureDef.expression,
                FormatString = measureDef.formatString ?? "General"
            };
            allMeasures.Add(md);
        }
    }
}

// UI
var form = new Form();
form.Text = "Select Measures to Import";
form.Size = new Size(1000, 600);
form.StartPosition = FormStartPosition.CenterScreen;

string currentFilter = "";
var searchBox = CreateSearchBox((s, e) => {
    currentFilter = ((TextBox)s).Text;
    ((Action)listView.Tag)?.Invoke();
});

var listView = CreateListView(allMeasures, () => currentFilter);
((Action)listView.Tag)?.Invoke();

var previewBox = new TextBox();
previewBox.Multiline = true;
previewBox.ScrollBars = ScrollBars.Vertical;
previewBox.ReadOnly = true;
previewBox.Size = new Size(960, 60);
previewBox.Location = new Point(10, 380);
listView.SelectedIndexChanged += (s, e) => {
    if (listView.SelectedItems.Count > 0)
        previewBox.Text = listView.SelectedItems[0].SubItems[1].Text;
};

var overwriteCheckbox = new CheckBox();
overwriteCheckbox.Text = "Overwrite existing measures";
overwriteCheckbox.Location = new Point(10, 450);
overwriteCheckbox.AutoSize = true;

var okButton = new Button();
okButton.Text = "Import Selected Measures";
okButton.DialogResult = DialogResult.OK;
okButton.Size = new Size(180, 30);
okButton.Location = new Point(580, 500);

var cancelButton = new Button();
cancelButton.Text = "Cancel";
cancelButton.DialogResult = DialogResult.Cancel;
cancelButton.Size = new Size(120, 30);
cancelButton.Location = new Point(780, 500);

form.Controls.Add(searchBox);
form.Controls.Add(listView);
form.Controls.Add(previewBox);
form.Controls.Add(overwriteCheckbox);
form.Controls.Add(okButton);
form.Controls.Add(cancelButton);
form.AcceptButton = okButton;
form.CancelButton = cancelButton;

if (form.ShowDialog() != DialogResult.OK)
    throw new Exception("User cancelled import.");

// Check for duplicates
var duplicateKeys = allMeasures
    .GroupBy(m => m.TableName + "|" + m.MeasureName)
    .Where(g => g.Count() > 1)
    .Select(g => g.Key)
    .ToHashSet();

// Import
foreach (ListViewItem item in listView.Items)
{
    if (item.Checked)
    {
        string measureName = item.SubItems[0].Text;
        string expression = item.SubItems[1].Text;
        string tableName = item.SubItems[2].Text;
        string report = item.SubItems[3].Text;
        var m = allMeasures.First(x => x.Report == report && x.TableName == tableName && x.MeasureName == measureName && x.Expression == expression);

        string key = m.TableName + "|" + m.MeasureName;
        bool isDuplicate = duplicateKeys.Contains(key);

        string actualTableName = m.TableName;
        string actualMeasureName = isDuplicate ? m.MeasureName + "_" + m.Report : m.MeasureName;

        var targetTable = Model.Tables.FirstOrDefault(t => t.Name == actualTableName);
        if (targetTable == null)
        {
            string calcTableDax = "DATATABLE(\"Dummy\", STRING, { { \"\" } })";
            targetTable = Model.AddCalculatedTable(actualTableName, calcTableDax);
            targetTable.IsHidden = true;
        }

        var existing = targetTable.Measures.FirstOrDefault(meas => meas.Name == actualMeasureName);
        if (existing != null)
        {
            if (overwriteCheckbox.Checked)
            {
                existing.Expression = expression;
                existing.FormatString = m.FormatString;
            }
        }
        else
        {
            var measure = targetTable.AddMeasure(actualMeasureName, expression);
            measure.FormatString = m.FormatString;
            measure.DisplayFolder = "Imported Measures";
        }
    }
}

Output("Selected local measures imported successfully from one or more reportExtensions.json files.");
